{% extends "registration/base.html" %}

{% load i18n %}

{% block main_content %}
  <div class="alert alert-info">
    {% blocktrans trimmed %}
      <p>We've emailed you instructions for setting your password, if an account exists with the email you entered. You should receive them shortly.</p>
      <p>If you don't receive an email, please make sure you've entered your primary address, and check your spam folder.</p>
    {% endblocktrans %}
  </div>
{% endblock %}
