{% extends "registration/base.html" %}

{% load i18n form_tags %}

{% block main_content %}
  {% if validlink %}
    <h3 class="text-center">{% trans "Change password" %}</h3>
    <form method="post" class="form-horizontal" style="margin-top: 20px">
      {% csrf_token %}
      {% render_form form %}
      <div class="col-sm-offset-4 col-sm-5">
        <button class="btn btn-primary col-lg-12" type="submit">{% trans "Submit" %}</button>
      </div>
    </form>
  {% else %}
    <div class="alert alert-danger">
      {% blocktrans trimmed %}
        The password reset link was invalid, possibly because it has already been used.
        Please request a new password reset.
      {% endblocktrans %}
    </div>
  {% endif %}
{% endblock %}
