{% load i18n %}
<th width="50%" class="sortable" data-sort_order="name">{% trans "Domain" %}</th>
<th class="sortable" data-sort_order="quota">{% trans "Limit" %}</th>
<th class="sortable" data-sort_order="quota_value__used">{% trans "Used" %}</th>
<th class="sortable" data-sort_order="allocated_quota">{% trans "Allocated" %}</th>
<th class="sortable" data-sort_order="quota_usage" id="quotabar">{% trans "Usage (%)" %}</th>
