{% extends "twocols.html" %}
{% load i18n lib_tags core_tags %}
{% load static %}

{% block pagetitle %}{% trans "Modoboa settings" %}{% endblock %}

{% block extra_css %}
<link rel="stylesheet" type="text/css" href="{% static 'css/jquery.sortable.css' %}" />
{% endblock %}

{% block extra_js %}
<script type="text/javascript" src="{% static 'js/jquery.infinitescroll.js' %}"></script>
<script type="text/javascript" src="{% static 'js/history.js' %}"></script>
<script type="text/javascript" src="{% static 'js/listing.js' %}"></script>
<script type="text/javascript" src="{% static 'js/twocols_nav.js' %}"></script>
<script type="text/javascript" src="{% static 'js/jquery.sortable.js' %}"></script>
<script type="text/javascript" src="{% static 'core/js/settings.js' %}"></script>
<script type="text/javascript">
$(document).ready(function() {
    var settings = new Settings({
        deflocation: "parameters/",
        formid: "settings_form",
        divid: "settings_content",
        reload_exceptions: ["extensions"],
        load_page_url: "{% url 'core:logs_page' %}"
    });
});
</script>
{% endblock %}

{% block leftcol %}{% admin_menu left_selection user %}{% endblock %}

{% block apparea %}<div id="settings_content"></div>{% endblock %}
