{% extends "registration/base.html" %}

{% load i18n %}

{% block main_content %}
  <div class="alert alert-success">
    {% url 'core:login' as login_url %}
    {% blocktrans with url=login_url trimmed %}
      Your password has been set. You may go ahead and <a href="{{ url }}">sign in</a> now.
    {% endblocktrans %}
  </div>
{% endblock %}
