<tbody>
  {% for row in table.rows %}
  <tr id="{{ row.id }}" class="{{ row.style }}" name="{{ row.trcpt }}">
    {% for cell in row.cols %}
    <td name="{{ cell.name }}" style="width: {{ cell.width }}; vertical-align: {{ cell.align }}"  class="{{ cell.cssclass }}">
    {% if cell.safe %}{{ cell.value|safe }}{% else %}{{ cell.value }}{% endif %}
    </td>
    {% endfor %}
  </tr>
  {% endfor %}
</tbody>