{% load i18n form_tags %}

{% if form.fields %}
  {% render_field form.create_dom_admin %}
  {% render_field form.dom_admin_username appended_text="@" %}
  {% render_field form.random_password %}
  {% render_field form.with_mailbox %}
  {% render_field form.create_aliases %}
{% else %}
  <div class="alert alert-warning">{% trans "You can't create a new domain administrator" %}</div>
{% endif %}
