# Generated by Django 1.10.5 on 2017-02-15 08:48
from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('relaydomains', '0005_auto_20161105_1426'),
    ]

    operations = [
        migrations.AlterField(
            model_name='relaydomain',
            name='last_modification',
            field=models.DateTimeField(auto_now=True, default=django.utils.timezone.now),
            preserve_default=False,
        ),
    ]
