{% extends "common/generic_modal_form.html" %}
{% load form_tags i18n %}
{% load static %}
{% block formbody %}
<div class="alert alert-info">
  <h4>{% trans "Help" %}</h4>
  {% block helptext %}{% endblock %}
</div>
{% render_form form %}
<iframe id="import_target" name="import_target" src="#" 
        style="width:0;height:0;border:0px solid #fff;"></iframe>
<div id="import_status" style="text-align: center;display:none">
  <img src="{% static 'css/spinner.gif' %}" /> {% trans "Importing..." %}
</div>
<div id="import_result"></div>
{% endblock %}
