#searchbox {
    margin-right: 10px;
    z-index: 2;
    position: relative;
}

#searchbox img {
    float: left;
    margin-top: 4px;
    margin-right: 5px;
}
  
#searchparams {
    background: white;
    padding: 5px;
    width: 90%;
    border-bottom: 5px solid #CCCCCC;
    border-left: 5px solid #CCCCCC;
    border-right: 5px solid #CCCCCC;
}