{% load i18n form_tags %}

<h2>{% trans "Profile" %} <small>{% trans "Update your personal information" %}</small></h2>
<hr>
<form class="form-horizontal" method="POST" action="{% url 'core:user_profile' %}"
      id="uprefs_form">
  {% csrf_token %}
  {% render_form form %}
  {% if show_password_url %}
    <div class="row">
      <div class="col-sm-offset-4 col-sm-8">
        <a href="{{ password_url }}" target="_blank">
          {% trans "Click here to update your password" %}
        </a>
      </div>
    </div>
  {% endif %}
  <div class="form-actions" style="margin-top: 15px;">
    <button type="submit" class="btn btn-primary btn-lg col-sm-offset-4 col-sm-4" id="update" name="update">{% trans 'Update' %}</button>
  </div>
</form>
