{% load lib_tags i18n admin_tags %}

{% for domain in domains %}
<tr>
  <td name="name"><a href="{% url 'admin:domain_detail' domain.id %}">{{ domain }}</a></td>
  <td>{{ domain.quota }}M</td>
  <td>{{ domain.used_quota }}M</td>
  <td>{{ domain.allocated_quota }}M</td>
  <td name="quota">
    <div class="progress">
      <div class="{% progress_color domain.used_quota_in_percent %}" style="width: {{ domain.used_quota_in_percent }}%" title="{{ domain.used_quota_in_percent }}% ({{ domain.used_quota }} {% trans "MB" %}) {% trans 'used' context 'quota' %}"></div>
      <div class="{% progress_color domain.allocated_quota_in_percent %}" style="width: {{ domain.allocated_quota_in_percent }}%" title="{{ domain.allocated_quota_in_percent }}% ({{ domain.allocated_quota }} {% trans "MB" %}) {% trans 'allocated' %}"></div>
    </div>
  </td>
</tr>
{% endfor %}
