/**
 * selectize.legacy.css (v0.12.4) - Default Theme
 * Copyright (c) 2013–2015 Brian Reavis & contributors
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain a copy of the License at:
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under
 * the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
 * ANY KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 *
 * @author Brian Reavis <brian@thirdroute.com>
 */

@import "selectize";

@selectize-font-size: 13px;
@selectize-line-height: 20px;

@selectize-color-input-full: #f2f2f2;
@selectize-color-item: #b8e76f;
@selectize-color-item-text: #3d5d18;
@selectize-color-item-border: #74b21e;
@selectize-color-item-active: #92c836;
@selectize-color-item-active-border: #6f9839;
@selectize-color-highlight: rgba(255,237,40,0.4);
@selectize-color-dropdown-item-active: #fffceb;
@selectize-color-dropdown-item-active-text: @selectize-color-text;
@selectize-color-optgroup: #f8f8f8;
@selectize-color-optgroup-text: @selectize-color-text;
@selectize-width-item-border: 1px;

@selectize-padding-x: 10px;
@selectize-padding-y: 10px;
@selectize-padding-item-x: 5px;
@selectize-padding-item-y: 1px;
@selectize-padding-dropdown-item-x: 10px;
@selectize-padding-dropdown-item-y: 7px;
@selectize-margin-item-x: 4px;
@selectize-margin-item-y: 4px;

.selectize-control {
	&.multi {
		.selectize-input [data-value] {
			text-shadow: 0 1px 0 rgba(255,255,255,0.1);
			.selectize-border-radius(3px);
			.selectize-vertical-gradient(#b8e76f, #a9e25c);
			.selectize-box-shadow(0 1px 1px rgba(0,0,0,0.1));
			&.active {
				.selectize-vertical-gradient(#92c836, #7abc2c);
			}
		}
	}
	&.single {
		.selectize-input {
			.selectize-box-shadow(~"inset 0 1px 0 rgba(255,255,255,0.8), 0 2px 0 #e0e0e0, 0 3px 0 #c8c8c8, 0 4px 1px rgba(0,0,0,0.1)");
			.selectize-vertical-gradient(#f5f5f5, #efefef);
		}
	}
}

.selectize-control.single .selectize-input, .selectize-dropdown.single {
	border-color: #b8b8b8;
}

.selectize-dropdown {
	.optgroup-header {
		font-weight: bold;
		font-size: 0.8em;
		border-bottom: 1px solid @selectize-color-dropdown-border-top;
		border-top: 1px solid @selectize-color-dropdown-border-top;
	}
}