{% extends "admin/header.html" %}{% load i18n admin_tags %}
{% load static %}

{% block extra_css %}
{{ block.super }}
<link rel="stylesheet" type="text/css" href="{% static 'css/jquery.sortable.css' %}" />
{% endblock %}

{% block extra_js %}
{{ block.super }}
<script type="text/javascript" src="{% static 'js/creation_wizard.js' %}"></script>
<script type="text/javascript" src="{% static 'js/dynamic_input.js' %}"></script>
<script type="text/javascript" src="{% static 'js/jquery.sortable.js' %}"></script>
<script type="text/javascript" src="{% static 'js/jquery.infinitescroll.js' %}"></script>
<script type="text/javascript">
var admin;
</script>
{% endblock %}

{% block leftcol %}
<ul class="nav nav-sidebar">
  {% block actions %}{% endblock %}
</ul>

{% get_extra_admin_content user "leftcol" selection %}
{% endblock %}

{% block apparea %}
<div class="row">
  <div class="col-sm-5">
    {% include "admin/searchform.html" %}
  </div>
  <div class="col-sm-3">
    {% block toolbar_right %}{% endblock %}
  </div>
  <div id="taglist" class="col-sm-4"> </div>
</div>

<div id="listing">
  <table id="objects_table" class="table {{ styles }}">
    <thead>
      <tr>
        {% block table_header %}{% endblock %}
      </tr>
    </thead>
    <tbody>
    </tbody>
  </table>
</div>
{% endblock %}
