{% load i18n %}
<th class="sortable" data-sort_order="queue_id">{% trans "Queue ID" %}</th>
<th class="sortable" data-sort_order="date">{% trans "Date" %}</th>
<th class="sortable" data-sort_order="sender">{% trans "From" %}</th>
<th class="sortable" data-sort_order="rcpt">{% trans "To" %}</th>
<th class="sortable" data-sort_order="status">{% trans "Status" %}</th>
