{% load form_tags %}

{% render_field form.email %}
{% if form.create_alias_with_old_address %}
  {% render_field form.create_alias_with_old_address %}
{% endif %}
{% include "common/generic_field.html" with field=form.quota help_display_mode="tooltip" label_width="col-sm-4" activator=True activator_value=form.quota_act.value deactivate_if_empty=False activator_label=_("Use domain default value") activator_size="col-sm-5" %}
{% render_field form.message_limit %}
{% render_fields_group form "aliases" %}
{% render_fields_group form "senderaddress" %}
