// Rubik Font

@import "~@fontsource/rubik/300.css";
@import "~@fontsource/rubik/300-italic.css";
@import "~@fontsource/rubik/400.css";
@import "~@fontsource/rubik/400-italic.css";
@import "~@fontsource/rubik/500.css";
@import "~@fontsource/rubik/500-italic.css";
@import "~@fontsource/rubik/700.css";
@import "~@fontsource/rubik/700-italic.css";

// Global styles

.m-label {
    display: block;
    margin-bottom: 4px;
    color: var(--v-label-base);
}
