{% load i18n %}
{% block table_header %}
  <th id="name" class="sortable" data-sort_order="name" width="">{% trans "Name" %}</th>
  <th id="aliases" width="">{% trans "Alias(es)" %}</th>
  <th id="tags" class="sortable" data-sort_order="tags" width="">{% trans "Tags" %}</th>
  {% if enable_mx_checks or enable_dnsbl_checks %}
    <th id="dns" width="">{% trans "DNS status" %}</th>
  {% endif %}
  <th id="actions" width="">{% trans "Actions" %}</th>
{% endblock %}
