(window["webpackJsonp"]=window["webpackJsonp"]||[]).push([["chunk-99c3e052"],{"2fa4":function(t,e,a){"use strict";a("20f6");var s=a("80d2");e["a"]=Object(s["j"])("spacer","div","v-spacer")},"4a00":function(t,e,a){"use strict";a.r(e);var s=function(){var t=this,e=t.$createElement,a=t._self._c||e;return a("div",[a("v-toolbar",{attrs:{flat:""}},[a("v-toolbar-title",[a("translate",[t._v("Message history")])],1)],1),a("message-list")],1)},n=[],i=function(){var t=this,e=t.$createElement,a=t._self._c||e;return a("v-card",{staticClass:"mt-6"},[a("v-toolbar",{attrs:{flat:""}},[a("v-spacer"),a("v-text-field",{attrs:{"prepend-inner-icon":"mdi-magnify",placeholder:t._f("translate")("Search"),filled:"",outlined:"",dense:"","hide-details":""},model:{value:t.search,callback:function(e){t.search=e},expression:"search"}})],1),a("v-data-table",{staticClass:"elevation-1",attrs:{headers:t.headers,items:t.messages,search:t.search,options:t.options,"server-items-length":t.total},on:{"update:options":function(e){t.options=e}},scopedSlots:t._u([{key:"item.date",fn:function(e){var a=e.item;return[t._v(" "+t._s(t._f("date")(a.date))+" ")]}},{key:"item.sender",fn:function(e){var a=e.item;return[t._v(" "+t._s(t._f("truncate")(a.sender,50))+" ")]}}])})],1)},r=[],o=(a("ac1f"),a("841c"),a("62ed")),c={data:function(){return{messages:[],headers:[{text:this.$gettext("Queue ID"),value:"queue_id"},{text:this.$gettext("Date"),value:"date",width:"20%"},{text:this.$gettext("Status"),value:"status"},{text:this.$gettext("To"),value:"rcpt"},{text:this.$gettext("From"),value:"sender"}],loading:!0,options:{},search:"",total:0}},methods:{fetchMessages:function(){var t,e=this;this.options.sortBy.length&&(t=(this.options.sortDesc[0]?"-":"")+this.options.sortBy[0]);var a={ordering:t,page:this.options.page,page_size:this.options.itemsPerPage,search:this.search};this.loading=!0,o["a"].getMessages(a).then((function(t){e.messages=t.data.results,e.total=t.data.count,e.loading=!1}))}},watch:{options:{handler:function(){this.fetchMessages()},deep:!0},search:function(t){this.fetchMessages()}}},l=c,u=a("2877"),d=a("6544"),h=a.n(d),f=a("b0af"),p=a("8fea"),g=a("2fa4"),v=a("8654"),m=a("71d9"),b=Object(u["a"])(l,i,r,!1,null,null,null),x=b.exports;h()(b,{VCard:f["a"],VDataTable:p["a"],VSpacer:g["a"],VTextField:v["a"],VToolbar:m["a"]});var _={components:{MessageList:x}},w=_,T=a("2a7f"),M=Object(u["a"])(w,s,n,!1,null,null,null);e["default"]=M.exports;h()(M,{VToolbar:m["a"],VToolbarTitle:T["a"]})},"62ed":function(t,e,a){"use strict";var s=a("3ffe"),n="logs";e["a"]={getAuditTrail:function(t){return s["a"].get("/".concat(n,"/audit-trail/"),{params:t})},getMessages:function(t){return s["a"].get("/".concat(n,"/messages/"),{params:t})}}}}]);
//# sourceMappingURL=chunk-99c3e052.9c0155f3.js.map