{% load i18n %}

{% autoescape off %}
  {% blocktrans with username=user.get_username trimmed %}
  To initiate the password reset process for your {{ username }} Modoboa account, click the link below:
  {% endblocktrans %}

  {{ protocol }}://{{ domain }}{% url 'password_reset_confirm' uidb64=uid token=token %}

  {% blocktrans trimmed %}
  If clicking the link above doesn't work, please copy and paste the URL in a new browser window instead.
  {% endblocktrans %}

  {% blocktrans trimmed %}
  Sincerely, The Modoboa Team.
  {% endblocktrans %}
{% endautoescape %}
