{% load i18n %}
{% for field in group %}
<div{% if forloop.first %} id="original_{{ pattern }}"{% endif %} class="form-group {% if haserror %} error{% endif %}">
  <label class="control-label col-lg-4 col-md-4 col-sm-4">
    {% if forloop.first %}
      {{ label }}
      {% if help_text %}
    <a href="#" class="help" data-original-title="{% trans 'Help:' %} {{ label }}" data-content="{{ help_text }}" rel="popover" tabindex="-1">?</a>
      {% endif %}
    {% endif %}
  </label>
  <div class="controls col-lg-5 col-md-5 col-sm-5">
    {{ field }}
    {% if field.errors %}
    <p class="help-block">
      {% for error in field.errors %}
      {{ error|safe }}
      {% endfor %}
    </p>
    {% endif %}
  </div>
</div>
{% endfor %}
