{% load i18n lib_tags %}
<ul class="nav nav-sidebar">
  <li class="nav-header">{% trans "Resources usage" %}</li>
  {% for l in limits %}
  <li class="resource">
    {{ l.label }} ({{ l }})
    <div rel="tooltip" title="{{ l.current_value }}/{{ l.max_value }}" class="progress">
      <div class="{% progress_color l.usage %}" style="width: {{ l }}" aria-valuemin="0" aria-valuemax="100"></div>
    </div>
  </li>
  {% endfor %}
</ul>
