{% load core_tags %}

{% for l in logs %}
<tr>
  <td>{{ l.date_created|date:"SHORT_DATETIME_FORMAT" }}</td>
  <td>{{ l.level|colorize_level|safe }}</td>
  <td>{{ l.logger }}</td>
  <td>{{ l.message|tohtml|safe }}</td>
</tr>
{% endfor %}
