<!DOCTYPE html>
{% load static %}
<html lang="en">
  <head>
    <meta charset="utf-8" />
    <link rel="stylesheet" type="text/css" href="{% static 'css/viewmail.css' %}" />
  </head>
  <body>
    {% if headers %}
      <div id="table-container">
        {{ headers }}
      </div>
    {% endif %}
    {{ mailbody|safe }}
  </body>
</html>
