{% extends "registration/base.html" %}

{% load i18n form_tags %}

{% block main_content %}

  {% if announcement %}
    <div class="row">
      <div class="col-xs-12 col-sm-offset-3 col-sm-7">
        <div class="alert alert-warning text-center">
          {{ announcement }}
        </div>
      </div>
    </div>
  {% endif %}

  <h3 class="text-center">{% trans "Forgot password?" %}</h3>
  <form class="form-horizontal" method="post" style="margin-top: 20px">
    {% csrf_token %}
    {% render_field form.email %}
    <div class="row">
      <div class="col-sm-offset-4 col-sm-5">
        <div class="alert alert-info">
          {% trans "Please fill-in your primary email address" %}
        </div>
      </div>
    </div>
    <div class="col-sm-offset-4 col-sm-5">
      <button type="submit" class="btn btn-primary col-lg-12">{% trans "Submit" %}</button>
    </div>
  </form>

{% endblock %}
