{% load i18n %}
<h2>{% trans "Preferences" %} <small>{% trans "Customize Modoboa" %}</small></h2>
<hr>
{% if forms %}
  <form class="form-horizontal" method="POST" action="{% url 'core:user_preferences' %}" 
        id="parameters_form">
    {% csrf_token %}
    {% include "parameters/base_form.html" %}
    <hr>
    <button type="submit" class="btn btn-primary col-xs-offset-4 col-xs-4" name="update">{% trans 'Save' %}</button>
  </form>
{% else %}
  <div class="alert alert-info">
    {% trans "No preferences available." %}
  </div>
{% endif %}
