{% extends "common/generic_modal.html" %}

{% load i18n %}

{% block modalbody %}
  {% if object.is_valid %}
    <div class="alert alert-success">
      {% trans "A DNS record has been found and is valid." %}
    </div>
  {% else %}
    <div class="alert alert-warning">
      {% trans "A DNS record has been found but its syntax is not valid:" %}
      {{ object.error }}
    </div>
  {% endif %}
{% endblock %}
