.menu[data-v-60c6ce22]{display:inline-block;position:fixed;margin-top:-20px;padding:5px 0;box-shadow:0 1px 5px 1px rgba(0,0,0,.3);overflow-y:auto;overflow-x:hidden;contain:content;z-index:99}.list[data-v-60c6ce22]{width:100%;height:100%;font-size:1.1em;overflow-x:hidden;padding:0}.selected[data-v-60c6ce22]{background-color:#fff8dc}.quota[data-v-1bab3872]{max-width:50%}.choice[data-v-65541534]{flex-basis:200px;background-color:#f2f5f7;border:1px solid #dbdddf;cursor:pointer}.choice--selected[data-v-65541534]{border-color:#046bf8!important}.choice--disabled[data-v-65541534]{cursor:unset;opacity:.5}.label--disabled[data-v-65541534]{opacity:.5}