# Generated by Django 1.10.7 on 2017-12-20 14:17
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('transport', '0001_initial'),
        ('admin', '0010_auto_20170215_1328'),
    ]

    operations = [
        migrations.AddField(
            model_name='domain',
            name='transport',
            field=models.OneToOneField(null=True, on_delete=django.db.models.deletion.SET_NULL, to='transport.Transport'),
        ),
    ]
