{% extends "fluid.html" %}

{% load core_tags %}

{% block container_content %}
  <div id="leftcol" class="sidebar collapse navbar-collapse">
    {% block leftcol %}{% endblock %}
  </div>
  <div class="main">
    {% block apparea %}{% endblock %}
  </div>
{% endblock %}

{% block extra_js %}
  {% if messages %}{% display_messages messages %}{% endif %}
{% endblock %}
