(window["webpackJsonp"]=window["webpackJsonp"]||[]).push([["chunk-13071ca0"],{"06d4":function(t,n,e){"use strict";e.r(n);var a=function(){var t=this,n=t.$createElement,e=t._self._c||n;return e("div",[e("span",{staticClass:"text-h4"},[e("translate",[t._v("Edit domain")])],1),e("div",{staticClass:"mt-4"}),e("domain-form",{attrs:{domain:t.domain}})],1)},r=[],c=e("458e"),o=e("8d28"),i={components:{DomainForm:o["a"]},data:function(){return{domain:{}}},mounted:function(){var t=this;c["a"].getDomain(this.$route.params.id).then((function(n){t.domain=n.data}))}},u=i,l=e("2877"),s=Object(l["a"])(u,a,r,!1,null,null,null);n["default"]=s.exports},"0fd9":function(t,n,e){"use strict";var a=e("ade3"),r=e("5530"),c=(e("caad"),e("2532"),e("99af"),e("b64b"),e("ac1f"),e("5319"),e("4ec9"),e("d3b7"),e("3ca3"),e("ddb0"),e("159b"),e("4b85"),e("2b0e")),o=e("d9f7"),i=e("80d2"),u=["sm","md","lg","xl"],l=["start","end","center"];function s(t,n){return u.reduce((function(e,a){return e[t+Object(i["H"])(a)]=n(),e}),{})}var f=function(t){return[].concat(l,["baseline","stretch"]).includes(t)},d=s("align",(function(){return{type:String,default:null,validator:f}})),b=function(t){return[].concat(l,["space-between","space-around"]).includes(t)},g=s("justify",(function(){return{type:String,default:null,validator:b}})),p=function(t){return[].concat(l,["space-between","space-around","stretch"]).includes(t)},v=s("alignContent",(function(){return{type:String,default:null,validator:p}})),j={align:Object.keys(d),justify:Object.keys(g),alignContent:Object.keys(v)},y={align:"align",justify:"justify",alignContent:"align-content"};function m(t,n,e){var a=y[t];if(null!=e){if(n){var r=n.replace(t,"");a+="-".concat(r)}return a+="-".concat(e),a.toLowerCase()}}var O=new Map;n["a"]=c["a"].extend({name:"v-row",functional:!0,props:Object(r["a"])(Object(r["a"])(Object(r["a"])({tag:{type:String,default:"div"},dense:Boolean,noGutters:Boolean,align:{type:String,default:null,validator:f}},d),{},{justify:{type:String,default:null,validator:b}},g),{},{alignContent:{type:String,default:null,validator:p}},v),render:function(t,n){var e=n.props,r=n.data,c=n.children,i="";for(var u in e)i+=String(e[u]);var l=O.get(i);return l||function(){var t,n;for(n in l=[],j)j[n].forEach((function(t){var a=e[t],r=m(n,t,a);r&&l.push(r)}));l.push((t={"no-gutters":e.noGutters,"row--dense":e.dense},Object(a["a"])(t,"align-".concat(e.align),e.align),Object(a["a"])(t,"justify-".concat(e.justify),e.justify),Object(a["a"])(t,"align-content-".concat(e.alignContent),e.alignContent),t)),O.set(i,l)}(),t(e.tag,Object(o["a"])(r,{staticClass:"row",class:l}),c)}})},"62ad":function(t,n,e){"use strict";var a=e("ade3"),r=e("5530"),c=(e("a9e3"),e("b64b"),e("ac1f"),e("5319"),e("4ec9"),e("d3b7"),e("3ca3"),e("ddb0"),e("caad"),e("159b"),e("2ca0"),e("4b85"),e("2b0e")),o=e("d9f7"),i=e("80d2"),u=["sm","md","lg","xl"],l=function(){return u.reduce((function(t,n){return t[n]={type:[Boolean,String,Number],default:!1},t}),{})}(),s=function(){return u.reduce((function(t,n){return t["offset"+Object(i["H"])(n)]={type:[String,Number],default:null},t}),{})}(),f=function(){return u.reduce((function(t,n){return t["order"+Object(i["H"])(n)]={type:[String,Number],default:null},t}),{})}(),d={col:Object.keys(l),offset:Object.keys(s),order:Object.keys(f)};function b(t,n,e){var a=t;if(null!=e&&!1!==e){if(n){var r=n.replace(t,"");a+="-".concat(r)}return"col"!==t||""!==e&&!0!==e?(a+="-".concat(e),a.toLowerCase()):a.toLowerCase()}}var g=new Map;n["a"]=c["a"].extend({name:"v-col",functional:!0,props:Object(r["a"])(Object(r["a"])(Object(r["a"])(Object(r["a"])({cols:{type:[Boolean,String,Number],default:!1}},l),{},{offset:{type:[String,Number],default:null}},s),{},{order:{type:[String,Number],default:null}},f),{},{alignSelf:{type:String,default:null,validator:function(t){return["auto","start","end","center","baseline","stretch"].includes(t)}},tag:{type:String,default:"div"}}),render:function(t,n){var e=n.props,r=n.data,c=n.children,i=(n.parent,"");for(var u in e)i+=String(e[u]);var l=g.get(i);return l||function(){var t,n;for(n in l=[],d)d[n].forEach((function(t){var a=e[t],r=b(n,t,a);r&&l.push(r)}));var r=l.some((function(t){return t.startsWith("col-")}));l.push((t={col:!r||!e.cols},Object(a["a"])(t,"col-".concat(e.cols),e.cols),Object(a["a"])(t,"offset-".concat(e.offset),e.offset),Object(a["a"])(t,"order-".concat(e.order),e.order),Object(a["a"])(t,"align-self-".concat(e.alignSelf),e.alignSelf),t)),g.set(i,l)}(),t(e.tag,Object(o["a"])(r,{class:l}),c)}})}}]);
//# sourceMappingURL=chunk-13071ca0.50a54e3a.js.map