{% extends "admin/listing.html" %}

{% load i18n admin_tags lib_tags static%}

{% block pagetitle %}{% trans "Domains" %}{% endblock %}

{% block extra_css %}
  {{ block.super }}
  {% extra_static_content "domains" "css" user %}
{% endblock %}

{% block extra_js %}
  {{ block.super }}
  <script src="{% static 'transport/js/transport.js' %}" type="text/javascript"></script>
  <script type="text/javascript">
   $(document).ready(function() {
       admin = new Domains({
           load_page_url: "{% url 'admin:domain_page' %}"
       });
   });
  </script>
{% extra_static_content "domains" "js" user %}
{% endblock %}

{% block toolbar_right %}
  {% if perms.admin.add_domain %}
    <a href="{% url 'admin:domain_add' %}" class="btn btn-primary" data-toggle="ajaxmodal" modalcb="admin.newdomain_cb">
      <span class="fa fa-plus"></span> {% trans "Add" %}
    </a>
  {% endif %}
{% endblock %}

{% block actions %}{% domains_menu "domains" user %}{% endblock %}
