{% load i18n admin_tags lib_tags %}

{% for domain in domains %}
<tr id="{{ domain.id }}" class="{% if not domain.enabled %}text-muted{% endif %}" name="{{ forloop.counter }}">
  <td name="name"><a href="{% url 'admin:domain_detail' domain.id %}">{{ domain }}</a>{% if not domain.enabled %} ({% trans "disabled" %}){% endif %}</td>
  <td name="aliases">{% domain_aliases domain %} </td>
  <td name="tags">{% render_tags domain.tags %}</td>
  {% if enable_mx_checks or enable_dnsbl_checks %}
    <td name="dns">
      <small>
        {% include "admin/_domain_dnschecks_status.html" %}
      </small>
    </td>
  {% endif %}
  <td name="actions">{% domain_actions user domain %}</td>
</tr>
{% endfor %}
