.highligth{background-color:#515d78}.v-text-field--outlined[data-v-a8cd6aa0] fieldset{border-color:#bfc5d2}.v-input--checkbox[data-v-a8cd6aa0] .v-label{font-size:.875rem!important}.v-toolbar[data-v-eaa2cca2]{background-color:#f7f8fa!important}