# Generated by Django 2.2.10 on 2020-04-29 12:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0020_auto_20200421_0851'),
    ]

    operations = [
        migrations.AddField(
            model_name='localconfig',
            name='need_dovecot_update',
            field=models.BooleanField(default=False),
        ),
    ]
