<table id="{{ tableid }}">
  <thead>
    <tr>
      {% for c in table.columns %}
      <th id="{{ c.name }}" class="sortable" width="{{ c.width }}">
        {{ c|safe }}
        <span style="float: right"><img src="" /></span>
      </th>
      {% endfor %}
    </tr>
  </thead>

  <tbody>
    {% for row in table.rows %}
    <div style="position:relative">
    <tr id="{{ row.id }}" class="{% cycle 'odd' 'even' %}">
      {% for cell in row.cols %}
      {% if cell.safe %}
      <td width="{{ c.width }}">{{ cell.value|safe }}</td>
      {% else %}
      <td width="{{ c.width }}" class="{{ cell.class }}">{{ cell.value }}</td>
      {% endif %}
      {% endfor %}
    </tr>
    </div>
    {% endfor %}
  </tbody>
</table>
