# -*- coding: utf-8 -*-
# Generated by Django 1.11.18 on 2020-01-21 15:59
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0018_auto_20200104_1714'),
    ]

    operations = [
        migrations.CreateModel(
            name='ExtensionUpdateHistory',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('extension', models.CharField(max_length=100)),
                ('version', models.CharField(max_length=30)),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='extensionupdatehistory',
            unique_together=set([('extension', 'version')]),
        ),
    ]
