{% extends "common/generic_modal.html" %}

{% load form_tags %}

{% block modalbody %}
  {% if error %}<div class="alert alert-danger" role="alert">{{ error }}</div>{% endif %}
<form class="form-horizontal" action="{{ action }}" enctype="{{ enctype }}" 
      method="POST" id="{{ formid }}" name="{{ formid }}" role="form"
      target="{{ target }}">
  {% csrf_token %}
  {% block formbody %}
  {% render_form form %}
  {% block extrafields %}{% endblock %}
  {% endblock %}
</form>
{% endblock %}
