{% extends "twocols.html" %}{% load i18n core_tags lib_tags %}
{% load static %}

{% block pagetitle %}{% trans "Settings" %}{% endblock %}

{% block extra_css %}
<link rel="stylesheet" type="text/css" href="{% static 'core/css/userprefs.css' %}" />
{% endblock %}

{% block extra_js %}
<script type="text/javascript" src="{% static 'js/history.js' %}"></script>
<script type="text/javascript" src="{% static 'js/listing.js' %}"></script>
<script type="text/javascript" src="{% static 'js/twocols_nav.js' %}"></script>
<script type="text/javascript" src="{% static 'ckeditor/ckeditor/ckeditor.js' %}"></script>
<script type="text/javascript">
 {% extra_static_content "user_index" "js" user %}

 function preferencesCallback() {
     $(document).trigger('preferencesLoaded');
 }

 $(document).ready(function() {
     var userprefs = new TwocolsNav({
         deflocation: "profile/",
         formid: "uprefs_form",
         divid: "uprefs_content",
         reload_exceptions: ["profile", "api"]
     });
 });
</script>
{% endblock %}

{% block leftcol %}{% uprefs_menu left_selection user %}{% endblock %}

{% block apparea %}<div id="uprefs_content"></div>{% endblock %}
