{% load i18n admin_tags lib_tags %}

{% for ident in identities %}
  {% check_identity_status ident as identity_is_enabled %}
<tr id="{{ ident.id }}" class="{% if not identity_is_enabled %}text-muted{% endif %}" name="{{ forloop.counter }}">
  <td name="identity">
    <a href="{{ ident.get_absolute_url }}">{{ ident }}</a>{% if not identity_is_enabled %} {% trans "(disabled)" %}{% endif %}
  </td>
  <td name="name_or_rcpt">{{ ident.name_or_rcpt }}</td>
  <td name="tags">{% render_tags ident.tags %}</td>
  <td name="actions">{% identity_actions user ident %}</td>
</tr>
{% endfor %}
