{% load i18n %}

<div class="modal-dialog modal-lg">
  <div class="modal-content">
    <div class="modal-header">
      <button type="button" class="close" data-dismiss="modal"><span aria-hidden="true">&times;</span><span class="sr-only">{% trans "Close" %}</span></button>
      <h3 class="modal-title">{% blocktrans trimmed with domain=domain.name %}DKIM public key for {{ domain }}{% endblocktrans %}</h3>
    </div>
    <div class="modal-body">
      <div class="alert alert-info" style="overflow-wrap: break-word;">
        <strong>{% trans "Raw format" %}</strong><br>
        {{ domain.dkim_public_key }}
      </div>
      <div class="alert alert-info" style="overflow-wrap: break-word;">
        <strong>{% trans "Bind/named format" %}</strong><br>
        <span style="white-space: pre">{{ domain.bind_format_dkim_public_key }}</span>
      </div>
    </div>
    <div class="modal-footer">
      <a class="btn btn-default" data-dismiss="modal">{% trans "Close" %}</a>
    </div>
  </div>
</div>
