{% extends "admin/importform.html" %}

{% load i18n %}

{% block helptext %}
  {% blocktrans trimmed %}
    Provide a CSV file where lines respect one of the following formats:
    <ul>
      <li><em>domain; name; quota; default mailbox quota; enabled</em></li>
      <li><em>domainalias; name; targeted domain; enabled</em></li>
      <li><em>relaydomain; name; target host; target port; service; enabled; verify recipients</em></li>
    </ul>
    <p>The first element of each line is mandatory and must be equal to one of the previous values.</p>
    <p>You can use a different character as separator.</p>
  {% endblocktrans %}
{% endblock %}
