{% load core_tags i18n %}

<!-- Current features to sponsor -->
<div class="panel panel-primary">
  <div class="panel-heading">
    <h3 class="panel-title">{% trans "Features looking for sponsoring" %}</h3>
  </div>
  <div class="panel-body">
    {% for feature in features %}
      <h4>{{ feature.name }}
        <span class="pull-right">
          <span class="label label-success">{% trans "Goal:" %} {{ feature.goal|currencyfmt }}</span>
        </span>
      </h4>
      <p class="slogan">{{ feature.slogan|safe }}</p>
      <div class="text-center">
        <a href="{{ feature.full_url }}" class="btn btn-primary" target="_blank">
          {% trans "More" %}</a>
      </div>
      {% if not forloop.last %}<hr>{% endif %}
    {% empty %}
      <div class="alert alert-info">{% trans "Nothing to sponsor yet." %}</div>
    {% endfor %}
  </div>
</div>
