{% load i18n %}
<div id="searchbox">
  <div class="input-group">
    <input type="text" class="form-control" name="searchfield"
           id="searchfield" placeholder="{% trans "Search..." %}" />
    <ul class="dropdown-menu pull-right" id="searchparams">
      <li>
        <div class="radio">
          <label>
            <input type="radio" id="crit_from_addr" name="scriteria" value="from_addr" checked> {% trans "From address" %}
          </label>
        </div>
      </li>
      <li>
        <div class="radio">
          <label>
            <input type="radio" id="crit_subject" name="scriteria" value="subject"> {% trans "Subject" %}
          </label>
        </div>
      </li>
      {% for opt in extraopts %}
      <li>
        <div class="radio">
          <label>
            <input type="radio" id="crit_{{ opt.name }}" name="scriteria" value="{{ opt.name }}">{{ opt.label }}
          </label>
        </div>
      </li>
      {% endfor %}
      <li>
        <div class="radio">
          <label>
            <input type="radio" id="crit_both" name="scriteria" value="both">{% trans "Both" %}
          </label>
        </div>
      </li>
    </ul>
    <a href="#" class="input-group-addon" data-toggle="dropdown"><span class="fa fa-search" id="searchopts"></span></a>
  </div>
</div>
