{% load static %}
<thead>
  <tr>
    {% for c in table.columns %}
    {% if c.sortable %}
    <th id="{{ c.name }}" class="sortable" data-sort_order="{{ c.sort_order }}" width="{{ c.width }}" align="{{ c.align }}">
    {% else %}
    <th id="{{ c.name }}" width="{{ c.width }}">
    {% endif %}
      {{ c|safe }}
      <span style="float: right"><img style="display: none" alt="" src="{% static 'pics/transparent.png' %}" /></span>
    </th>
    {% endfor %}
  </tr>
</thead>
