# Generated by Django 1.10.7 on 2017-11-24 14:08
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('relaydomains', '0008_auto_20171123_1653'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='relaydomain',
            name='domain',
        ),
        migrations.RemoveField(
            model_name='relaydomain',
            name='service',
        ),
        migrations.DeleteModel(
            name='RelayDomain',
        ),
        migrations.DeleteModel(
            name='Service',
        ),
    ]
