{% load i18n %}

<div class="panel panel-primary">
  <div class="panel-heading"><h3 class="panel-title">{% trans "Global statistics" %}</h3></div>
  <table class="table">
    <tr>
      <th>{% trans "Type" %}</th>
      <th>{% trans "Quantity" %}</th>
    </tr>
    <tr>
      <td>{% trans "Domains" %}</td><td>{{ domains_counter }}</td>
    </tr>
    <tr>
      <td>{% trans "Domain aliases" %}</td><td>{{ domain_aliases_counter }}</td>
    </tr>
    <tr>
      <td>{% trans "Identities" %}</td><td>{{ identities_counter }}</td>
    </tr>
  </table>
</div>
