# Generated by Django 1.10.5 on 2017-02-15 08:48
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('admin', '0008_domain_enable_dns_checks'),
    ]

    operations = [
        migrations.RenameField(
            model_name='domain',
            old_name='quota',
            new_name='default_mailbox_quota',
        ),
    ]
