{% load i18n form_tags %}
{% render_field form.name %}
{% render_field form.type %}
{% render_field form.quota %}
{% render_field form.default_mailbox_quota %}
{% render_field form.message_limit %}
{% render_fields_group form "aliases" %}
{% render_field form.enabled %}
{% render_field form.enable_dns_checks %}
{% render_field form.enable_dkim %}
{% render_field form.dkim_key_selector %}
{% render_field form.dkim_key_length %}
