# Generated by Django 2.2.12 on 2020-10-16 09:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0021_localconfig_need_dovecot_update'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='tfa_enabled',
            field=models.BooleanField(default=False),
        ),
    ]
