{% load i18n %}
<th class="sortable" data-sort_order="name">{% trans "ID" %}</th>
<th class="sortable" data-sort_order="quota">{% trans "Date" %}</th>
<th class="sortable" data-sort_order="quota_value__used">{% trans "From" %}</th>
<th class="sortable" data-sort_order="allocated_quota">{% trans "To" %}</th>
<th class="sortable" data-sort_order="quota_usage" id="quotabar">{% trans "Status" %}</th>
