.v-toolbar[data-v-02e474f1]{background-color:#f7f8fa!important}.v-list-item--active[data-v-b9a79e62]{background-color:#034bad;color:#fff;opacity:1}.v-list-item--active[data-v-b9a79e62]:before{opacity:0}.user-box[data-v-b9a79e62]{background:rgba(0,0,0,.25)}.logo[data-v-b9a79e62]{cursor:pointer}.v-main[data-v-345ee5e5]{background-color:#f7f8fa}.m-label{display:block;margin-bottom:4px;color:var(--v-label-base)}