{% load i18n %}
<h2>{% trans "Logs" %} <small>{% trans "Previously on Modoboa..." %}</small></h2><hr>
<table id="logs_table" class="table">
  <thead>
    <tr>
      <th class="sortable" data-sort_order="date_created" width="20%">{% trans "Date" %}</th>
      <th class="sortable" data-sort_order="level" width="7%">{% trans "Level" %}</th>
      <th class="sortable" data-sort_order="logger" width="15%">{% trans "Logger" %}</th>
      <th class="sortable" data-sort_order="message" width="">{% trans "Message" %}</th>
    </tr>
  </thead>
  <tbody>
    {% include "core/logs_page.html" %}
  </tbody>
</table>
